/* SPDX-License-Identifier: GPL-2.0+ */
/*
 *  (C) Copyright 2010,2011
 *  NVIDIA Corporation <www.nvidia.com>
 */

#ifndef _TEGRA_PMU_H_
#define _TEGRA_PMU_H_

/* Set core and CPU voltages to nominal levels */
int pmu_set_nominal(void);

#endif	/* _TEGRA_PMU_H_ */
