/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2016 Andreas Färber
 */

#ifndef __CONFIGS_GEEKBOX_H
#define __CONFIGS_GEEKBOX_H

#include <configs/rk3368_common.h>

#define CONFIG_CONSOLE_SCROLL_LINES		10

#endif
