// SPDX-License-Identifier: GPL-2.0+
/*
 * Espresso7420 board file
 * Copyright (C) 2016 Samsung Electronics
 * Thomas Abraham <thomas.ab@samsung.com>
 */

#include <common.h>

int exynos_init(void)
{
	return 0;
}
