/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2008 Extreme Engineering Solutions, Inc.
 */

#ifndef	__TIMESTAMP_H__
#define	__TIMESTAMP_H__

#ifndef DO_DEPS_ONLY
#include "generated/timestamp_autogenerated.h"
#endif

#endif	/* __TIMESTAMP_H__ */
