/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2011 The Chromium OS Authors.
 * (C) Copyright 2010,2011 NVIDIA Corporation <www.nvidia.com>
 */

#ifndef _NVIDIA_EMC_H_
#define _NVIDIA_EMC_H_

int board_emc_init(void);

#endif
