/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2016 Bin Meng <bmeng.cn@gmail.com>
 */

#ifndef _QUARK_IRQ_H_
#define _QUARK_IRQ_H_

#define PIRQA_APIC_IRQ	16
#define PIRQB_APIC_IRQ	17
#define PIRQC_APIC_IRQ	18
#define PIRQD_APIC_IRQ	19
#define PIRQE_APIC_IRQ	20
#define PIRQF_APIC_IRQ	21
#define PIRQG_APIC_IRQ	22
#define PIRQH_APIC_IRQ	23

#endif /* _QUARK_IRQ_H_ */
