#if 0
# SPDX SPDX-License-Identifier: GPL-2.0+
#
# Constants defining the host architecture in assembler, C, and make files.
# The values are arbitrary.
#
# Copyright 2019 Heinrich Schuchardt <xypron.glpk@gmx.de>
#endif

#if 0
export HOST_ARCH_AARCH64=0xaa64
export HOST_ARCH_ARM=0x00a7
export HOST_ARCH_RISCV32=0x5032
export HOST_ARCH_RISCV64=0x5064
export HOST_ARCH_X86=0x0386
export HOST_ARCH_X86_64=0x8664
#endif

#define HOST_ARCH_AARCH64 0xaa64
#define HOST_ARCH_ARM 0x00a7
#define HOST_ARCH_RISCV32 0x5032
#define HOST_ARCH_RISCV64 0x5064
#define HOST_ARCH_X86 0x0386
#define HOST_ARCH_X86_64 0x8664
