/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2013-2014 Synopsys, Inc. All rights reserved.
 */

#ifndef __ASM_ARC_SECTIONS_H
#define __ASM_ARC_SECTIONS_H

#include <asm-generic/sections.h>

extern ulong __ivt_start;
extern ulong __ivt_end;

#endif /* __ASM_ARC_SECTIONS_H */
