#define	TPS659411		0x0
#define TPS659412		0x1
#define TPS659413		0x2
#define TPS659414		0x3

/* I2C device address for pmic tps65941 */
#define TPS65941_I2C_ADDR	(0x12 >> 1)
#define TPS65941_LDO_NUM		4
#define TPS65941_BUCK_NUM		5

/* Drivers name */
#define TPS65941_LDO_DRIVER		"tps65941_ldo"
#define TPS65941_BUCK_DRIVER		"tps65941_buck"

#define TPS65941_BUCK_VOLT_MASK		0xFF
#define TPS65941_BUCK_VOLT_MAX_HEX	0xFF
#define TPS65941_BUCK_VOLT_MAX		3340000
#define TPS65941_BUCK_MODE_MASK		0x1

#define TPS65941_LDO_VOLT_MASK		0x3E
#define TPS65941_LDO_VOLT_MAX_HEX	0x3A
#define TPS65941_LDO_VOLT_MIN_HEX	0x4
#define TPS65941_LDO_VOLT_MAX		3300000
#define TPS65941_LDO_MODE_MASK		0x1
#define TPS65941_LDO_BYPASS_EN		0x80
#define TP65941_BUCK_CONF_SLEW_MASK	0x7
