/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2004-2007 Freescale Semiconductor, Inc.
 */

 /* This file is autogenerated. Do not change */
#define CURRBD		4
#define DCOUNT		6
#define DESTPTR		5
#define SRCPTR		7
