/* SPDX-License-Identifier: BSD-3-Clause */
/**********************************************************************
 * Copyright (C) 2012-2019 Cadence Design Systems, Inc.
 *
 * THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT
 *
 **********************************************************************
 */

#ifndef REG_LPDDR4_CTL_REGS_H_
#define REG_LPDDR4_CTL_REGS_H_

#include "lpddr4_ddr_controller_macros.h"
#include "lpddr4_pi_macros.h"
#include "lpddr4_data_slice_0_macros.h"
#include "lpddr4_data_slice_1_macros.h"
#include "lpddr4_data_slice_2_macros.h"
#include "lpddr4_data_slice_3_macros.h"
#include "lpddr4_address_slice_0_macros.h"
#include "lpddr4_phy_core_macros.h"

typedef struct __attribute__((packed)) lpddr4_ctlregs_s {
	volatile uint32_t DENALI_CTL_0;
	volatile uint32_t DENALI_CTL_1;
	volatile uint32_t DENALI_CTL_2;
	volatile uint32_t DENALI_CTL_3;
	volatile uint32_t DENALI_CTL_4;
	volatile uint32_t DENALI_CTL_5;
	volatile uint32_t DENALI_CTL_6;
	volatile uint32_t DENALI_CTL_7;
	volatile uint32_t DENALI_CTL_8;
	volatile uint32_t DENALI_CTL_9;
	volatile uint32_t DENALI_CTL_10;
	volatile uint32_t DENALI_CTL_11;
	volatile uint32_t DENALI_CTL_12;
	volatile uint32_t DENALI_CTL_13;
	volatile uint32_t DENALI_CTL_14;
	volatile uint32_t DENALI_CTL_15;
	volatile uint32_t DENALI_CTL_16;
	volatile uint32_t DENALI_CTL_17;
	volatile uint32_t DENALI_CTL_18;
	volatile uint32_t DENALI_CTL_19;
	volatile uint32_t DENALI_CTL_20;
	volatile uint32_t DENALI_CTL_21;
	volatile uint32_t DENALI_CTL_22;
	volatile uint32_t DENALI_CTL_23;
	volatile uint32_t DENALI_CTL_24;
	volatile uint32_t DENALI_CTL_25;
	volatile uint32_t DENALI_CTL_26;
	volatile uint32_t DENALI_CTL_27;
	volatile uint32_t DENALI_CTL_28;
	volatile uint32_t DENALI_CTL_29;
	volatile uint32_t DENALI_CTL_30;
	volatile uint32_t DENALI_CTL_31;
	volatile uint32_t DENALI_CTL_32;
	volatile uint32_t DENALI_CTL_33;
	volatile uint32_t DENALI_CTL_34;
	volatile uint32_t DENALI_CTL_35;
	volatile uint32_t DENALI_CTL_36;
	volatile uint32_t DENALI_CTL_37;
	volatile uint32_t DENALI_CTL_38;
	volatile uint32_t DENALI_CTL_39;
	volatile uint32_t DENALI_CTL_40;
	volatile uint32_t DENALI_CTL_41;
	volatile uint32_t DENALI_CTL_42;
	volatile uint32_t DENALI_CTL_43;
	volatile uint32_t DENALI_CTL_44;
	volatile uint32_t DENALI_CTL_45;
	volatile uint32_t DENALI_CTL_46;
	volatile uint32_t DENALI_CTL_47;
	volatile uint32_t DENALI_CTL_48;
	volatile uint32_t DENALI_CTL_49;
	volatile uint32_t DENALI_CTL_50;
	volatile uint32_t DENALI_CTL_51;
	volatile uint32_t DENALI_CTL_52;
	volatile uint32_t DENALI_CTL_53;
	volatile uint32_t DENALI_CTL_54;
	volatile uint32_t DENALI_CTL_55;
	volatile uint32_t DENALI_CTL_56;
	volatile uint32_t DENALI_CTL_57;
	volatile uint32_t DENALI_CTL_58;
	volatile uint32_t DENALI_CTL_59;
	volatile uint32_t DENALI_CTL_60;
	volatile uint32_t DENALI_CTL_61;
	volatile uint32_t DENALI_CTL_62;
	volatile uint32_t DENALI_CTL_63;
	volatile uint32_t DENALI_CTL_64;
	volatile uint32_t DENALI_CTL_65;
	volatile uint32_t DENALI_CTL_66;
	volatile uint32_t DENALI_CTL_67;
	volatile uint32_t DENALI_CTL_68;
	volatile uint32_t DENALI_CTL_69;
	volatile uint32_t DENALI_CTL_70;
	volatile uint32_t DENALI_CTL_71;
	volatile uint32_t DENALI_CTL_72;
	volatile uint32_t DENALI_CTL_73;
	volatile uint32_t DENALI_CTL_74;
	volatile uint32_t DENALI_CTL_75;
	volatile uint32_t DENALI_CTL_76;
	volatile uint32_t DENALI_CTL_77;
	volatile uint32_t DENALI_CTL_78;
	volatile uint32_t DENALI_CTL_79;
	volatile uint32_t DENALI_CTL_80;
	volatile uint32_t DENALI_CTL_81;
	volatile uint32_t DENALI_CTL_82;
	volatile uint32_t DENALI_CTL_83;
	volatile uint32_t DENALI_CTL_84;
	volatile uint32_t DENALI_CTL_85;
	volatile uint32_t DENALI_CTL_86;
	volatile uint32_t DENALI_CTL_87;
	volatile uint32_t DENALI_CTL_88;
	volatile uint32_t DENALI_CTL_89;
	volatile uint32_t DENALI_CTL_90;
	volatile uint32_t DENALI_CTL_91;
	volatile uint32_t DENALI_CTL_92;
	volatile uint32_t DENALI_CTL_93;
	volatile uint32_t DENALI_CTL_94;
	volatile uint32_t DENALI_CTL_95;
	volatile uint32_t DENALI_CTL_96;
	volatile uint32_t DENALI_CTL_97;
	volatile uint32_t DENALI_CTL_98;
	volatile uint32_t DENALI_CTL_99;
	volatile uint32_t DENALI_CTL_100;
	volatile uint32_t DENALI_CTL_101;
	volatile uint32_t DENALI_CTL_102;
	volatile uint32_t DENALI_CTL_103;
	volatile uint32_t DENALI_CTL_104;
	volatile uint32_t DENALI_CTL_105;
	volatile uint32_t DENALI_CTL_106;
	volatile uint32_t DENALI_CTL_107;
	volatile uint32_t DENALI_CTL_108;
	volatile uint32_t DENALI_CTL_109;
	volatile uint32_t DENALI_CTL_110;
	volatile uint32_t DENALI_CTL_111;
	volatile uint32_t DENALI_CTL_112;
	volatile uint32_t DENALI_CTL_113;
	volatile uint32_t DENALI_CTL_114;
	volatile uint32_t DENALI_CTL_115;
	volatile uint32_t DENALI_CTL_116;
	volatile uint32_t DENALI_CTL_117;
	volatile uint32_t DENALI_CTL_118;
	volatile uint32_t DENALI_CTL_119;
	volatile uint32_t DENALI_CTL_120;
	volatile uint32_t DENALI_CTL_121;
	volatile uint32_t DENALI_CTL_122;
	volatile uint32_t DENALI_CTL_123;
	volatile uint32_t DENALI_CTL_124;
	volatile uint32_t DENALI_CTL_125;
	volatile uint32_t DENALI_CTL_126;
	volatile uint32_t DENALI_CTL_127;
	volatile uint32_t DENALI_CTL_128;
	volatile uint32_t DENALI_CTL_129;
	volatile uint32_t DENALI_CTL_130;
	volatile uint32_t DENALI_CTL_131;
	volatile uint32_t DENALI_CTL_132;
	volatile uint32_t DENALI_CTL_133;
	volatile uint32_t DENALI_CTL_134;
	volatile uint32_t DENALI_CTL_135;
	volatile uint32_t DENALI_CTL_136;
	volatile uint32_t DENALI_CTL_137;
	volatile uint32_t DENALI_CTL_138;
	volatile uint32_t DENALI_CTL_139;
	volatile uint32_t DENALI_CTL_140;
	volatile uint32_t DENALI_CTL_141;
	volatile uint32_t DENALI_CTL_142;
	volatile uint32_t DENALI_CTL_143;
	volatile uint32_t DENALI_CTL_144;
	volatile uint32_t DENALI_CTL_145;
	volatile uint32_t DENALI_CTL_146;
	volatile uint32_t DENALI_CTL_147;
	volatile uint32_t DENALI_CTL_148;
	volatile uint32_t DENALI_CTL_149;
	volatile uint32_t DENALI_CTL_150;
	volatile uint32_t DENALI_CTL_151;
	volatile uint32_t DENALI_CTL_152;
	volatile uint32_t DENALI_CTL_153;
	volatile uint32_t DENALI_CTL_154;
	volatile uint32_t DENALI_CTL_155;
	volatile uint32_t DENALI_CTL_156;
	volatile uint32_t DENALI_CTL_157;
	volatile uint32_t DENALI_CTL_158;
	volatile uint32_t DENALI_CTL_159;
	volatile uint32_t DENALI_CTL_160;
	volatile uint32_t DENALI_CTL_161;
	volatile uint32_t DENALI_CTL_162;
	volatile uint32_t DENALI_CTL_163;
	volatile uint32_t DENALI_CTL_164;
	volatile uint32_t DENALI_CTL_165;
	volatile uint32_t DENALI_CTL_166;
	volatile uint32_t DENALI_CTL_167;
	volatile uint32_t DENALI_CTL_168;
	volatile uint32_t DENALI_CTL_169;
	volatile uint32_t DENALI_CTL_170;
	volatile uint32_t DENALI_CTL_171;
	volatile uint32_t DENALI_CTL_172;
	volatile uint32_t DENALI_CTL_173;
	volatile uint32_t DENALI_CTL_174;
	volatile uint32_t DENALI_CTL_175;
	volatile uint32_t DENALI_CTL_176;
	volatile uint32_t DENALI_CTL_177;
	volatile uint32_t DENALI_CTL_178;
	volatile uint32_t DENALI_CTL_179;
	volatile uint32_t DENALI_CTL_180;
	volatile uint32_t DENALI_CTL_181;
	volatile uint32_t DENALI_CTL_182;
	volatile uint32_t DENALI_CTL_183;
	volatile uint32_t DENALI_CTL_184;
	volatile uint32_t DENALI_CTL_185;
	volatile uint32_t DENALI_CTL_186;
	volatile uint32_t DENALI_CTL_187;
	volatile uint32_t DENALI_CTL_188;
	volatile uint32_t DENALI_CTL_189;
	volatile uint32_t DENALI_CTL_190;
	volatile uint32_t DENALI_CTL_191;
	volatile uint32_t DENALI_CTL_192;
	volatile uint32_t DENALI_CTL_193;
	volatile uint32_t DENALI_CTL_194;
	volatile uint32_t DENALI_CTL_195;
	volatile uint32_t DENALI_CTL_196;
	volatile uint32_t DENALI_CTL_197;
	volatile uint32_t DENALI_CTL_198;
	volatile uint32_t DENALI_CTL_199;
	volatile uint32_t DENALI_CTL_200;
	volatile uint32_t DENALI_CTL_201;
	volatile uint32_t DENALI_CTL_202;
	volatile uint32_t DENALI_CTL_203;
	volatile uint32_t DENALI_CTL_204;
	volatile uint32_t DENALI_CTL_205;
	volatile uint32_t DENALI_CTL_206;
	volatile uint32_t DENALI_CTL_207;
	volatile uint32_t DENALI_CTL_208;
	volatile uint32_t DENALI_CTL_209;
	volatile uint32_t DENALI_CTL_210;
	volatile uint32_t DENALI_CTL_211;
	volatile uint32_t DENALI_CTL_212;
	volatile uint32_t DENALI_CTL_213;
	volatile uint32_t DENALI_CTL_214;
	volatile uint32_t DENALI_CTL_215;
	volatile uint32_t DENALI_CTL_216;
	volatile uint32_t DENALI_CTL_217;
	volatile uint32_t DENALI_CTL_218;
	volatile uint32_t DENALI_CTL_219;
	volatile uint32_t DENALI_CTL_220;
	volatile uint32_t DENALI_CTL_221;
	volatile uint32_t DENALI_CTL_222;
	volatile uint32_t DENALI_CTL_223;
	volatile uint32_t DENALI_CTL_224;
	volatile uint32_t DENALI_CTL_225;
	volatile uint32_t DENALI_CTL_226;
	volatile uint32_t DENALI_CTL_227;
	volatile uint32_t DENALI_CTL_228;
	volatile uint32_t DENALI_CTL_229;
	volatile uint32_t DENALI_CTL_230;
	volatile uint32_t DENALI_CTL_231;
	volatile uint32_t DENALI_CTL_232;
	volatile uint32_t DENALI_CTL_233;
	volatile uint32_t DENALI_CTL_234;
	volatile uint32_t DENALI_CTL_235;
	volatile uint32_t DENALI_CTL_236;
	volatile uint32_t DENALI_CTL_237;
	volatile uint32_t DENALI_CTL_238;
	volatile uint32_t DENALI_CTL_239;
	volatile uint32_t DENALI_CTL_240;
	volatile uint32_t DENALI_CTL_241;
	volatile uint32_t DENALI_CTL_242;
	volatile uint32_t DENALI_CTL_243;
	volatile uint32_t DENALI_CTL_244;
	volatile uint32_t DENALI_CTL_245;
	volatile uint32_t DENALI_CTL_246;
	volatile uint32_t DENALI_CTL_247;
	volatile uint32_t DENALI_CTL_248;
	volatile uint32_t DENALI_CTL_249;
	volatile uint32_t DENALI_CTL_250;
	volatile uint32_t DENALI_CTL_251;
	volatile uint32_t DENALI_CTL_252;
	volatile uint32_t DENALI_CTL_253;
	volatile uint32_t DENALI_CTL_254;
	volatile uint32_t DENALI_CTL_255;
	volatile uint32_t DENALI_CTL_256;
	volatile uint32_t DENALI_CTL_257;
	volatile uint32_t DENALI_CTL_258;
	volatile uint32_t DENALI_CTL_259;
	volatile uint32_t DENALI_CTL_260;
	volatile uint32_t DENALI_CTL_261;
	volatile uint32_t DENALI_CTL_262;
	volatile uint32_t DENALI_CTL_263;
	volatile uint32_t DENALI_CTL_264;
	volatile uint32_t DENALI_CTL_265;
	volatile uint32_t DENALI_CTL_266;
	volatile uint32_t DENALI_CTL_267;
	volatile uint32_t DENALI_CTL_268;
	volatile uint32_t DENALI_CTL_269;
	volatile uint32_t DENALI_CTL_270;
	volatile uint32_t DENALI_CTL_271;
	volatile uint32_t DENALI_CTL_272;
	volatile uint32_t DENALI_CTL_273;
	volatile uint32_t DENALI_CTL_274;
	volatile uint32_t DENALI_CTL_275;
	volatile uint32_t DENALI_CTL_276;
	volatile uint32_t DENALI_CTL_277;
	volatile uint32_t DENALI_CTL_278;
	volatile uint32_t DENALI_CTL_279;
	volatile uint32_t DENALI_CTL_280;
	volatile uint32_t DENALI_CTL_281;
	volatile uint32_t DENALI_CTL_282;
	volatile uint32_t DENALI_CTL_283;
	volatile uint32_t DENALI_CTL_284;
	volatile uint32_t DENALI_CTL_285;
	volatile uint32_t DENALI_CTL_286;
	volatile uint32_t DENALI_CTL_287;
	volatile uint32_t DENALI_CTL_288;
	volatile uint32_t DENALI_CTL_289;
	volatile uint32_t DENALI_CTL_290;
	volatile uint32_t DENALI_CTL_291;
	volatile uint32_t DENALI_CTL_292;
	volatile uint32_t DENALI_CTL_293;
	volatile uint32_t DENALI_CTL_294;
	volatile uint32_t DENALI_CTL_295;
	volatile uint32_t DENALI_CTL_296;
	volatile uint32_t DENALI_CTL_297;
	volatile uint32_t DENALI_CTL_298;
	volatile uint32_t DENALI_CTL_299;
	volatile uint32_t DENALI_CTL_300;
	volatile uint32_t DENALI_CTL_301;
	volatile uint32_t DENALI_CTL_302;
	volatile uint32_t DENALI_CTL_303;
	volatile uint32_t DENALI_CTL_304;
	volatile uint32_t DENALI_CTL_305;
	volatile uint32_t DENALI_CTL_306;
	volatile uint32_t DENALI_CTL_307;
	volatile uint32_t DENALI_CTL_308;
	volatile uint32_t DENALI_CTL_309;
	volatile uint32_t DENALI_CTL_310;
	volatile uint32_t DENALI_CTL_311;
	volatile uint32_t DENALI_CTL_312;
	volatile uint32_t DENALI_CTL_313;
	volatile uint32_t DENALI_CTL_314;
	volatile uint32_t DENALI_CTL_315;
	volatile uint32_t DENALI_CTL_316;
	volatile uint32_t DENALI_CTL_317;
	volatile uint32_t DENALI_CTL_318;
	volatile uint32_t DENALI_CTL_319;
	volatile uint32_t DENALI_CTL_320;
	volatile uint32_t DENALI_CTL_321;
	volatile uint32_t DENALI_CTL_322;
	volatile uint32_t DENALI_CTL_323;
	volatile uint32_t DENALI_CTL_324;
	volatile uint32_t DENALI_CTL_325;
	volatile uint32_t DENALI_CTL_326;
	volatile uint32_t DENALI_CTL_327;
	volatile uint32_t DENALI_CTL_328;
	volatile uint32_t DENALI_CTL_329;
	volatile uint32_t DENALI_CTL_330;
	volatile uint32_t DENALI_CTL_331;
	volatile uint32_t DENALI_CTL_332;
	volatile uint32_t DENALI_CTL_333;
	volatile uint32_t DENALI_CTL_334;
	volatile uint32_t DENALI_CTL_335;
	volatile uint32_t DENALI_CTL_336;
	volatile uint32_t DENALI_CTL_337;
	volatile uint32_t DENALI_CTL_338;
	volatile uint32_t DENALI_CTL_339;
	volatile uint32_t DENALI_CTL_340;
	volatile uint32_t DENALI_CTL_341;
	volatile uint32_t DENALI_CTL_342;
	volatile uint32_t DENALI_CTL_343;
	volatile uint32_t DENALI_CTL_344;
	volatile uint32_t DENALI_CTL_345;
	volatile uint32_t DENALI_CTL_346;
	volatile uint32_t DENALI_CTL_347;
	volatile uint32_t DENALI_CTL_348;
	volatile uint32_t DENALI_CTL_349;
	volatile uint32_t DENALI_CTL_350;
	volatile uint32_t DENALI_CTL_351;
	volatile uint32_t DENALI_CTL_352;
	volatile uint32_t DENALI_CTL_353;
	volatile uint32_t DENALI_CTL_354;
	volatile uint32_t DENALI_CTL_355;
	volatile uint32_t DENALI_CTL_356;
	volatile uint32_t DENALI_CTL_357;
	volatile uint32_t DENALI_CTL_358;
	volatile uint32_t DENALI_CTL_359;
	volatile uint32_t DENALI_CTL_360;
	volatile uint32_t DENALI_CTL_361;
	volatile uint32_t DENALI_CTL_362;
	volatile uint32_t DENALI_CTL_363;
	volatile uint32_t DENALI_CTL_364;
	volatile uint32_t DENALI_CTL_365;
	volatile uint32_t DENALI_CTL_366;
	volatile uint32_t DENALI_CTL_367;
	volatile uint32_t DENALI_CTL_368;
	volatile uint32_t DENALI_CTL_369;
	volatile uint32_t DENALI_CTL_370;
	volatile uint32_t DENALI_CTL_371;
	volatile uint32_t DENALI_CTL_372;
	volatile uint32_t DENALI_CTL_373;
	volatile uint32_t DENALI_CTL_374;
	volatile uint32_t DENALI_CTL_375;
	volatile uint32_t DENALI_CTL_376;
	volatile uint32_t DENALI_CTL_377;
	volatile uint32_t DENALI_CTL_378;
	volatile uint32_t DENALI_CTL_379;
	volatile uint32_t DENALI_CTL_380;
	volatile uint32_t DENALI_CTL_381;
	volatile uint32_t DENALI_CTL_382;
	volatile uint32_t DENALI_CTL_383;
	volatile uint32_t DENALI_CTL_384;
	volatile uint32_t DENALI_CTL_385;
	volatile uint32_t DENALI_CTL_386;
	volatile uint32_t DENALI_CTL_387;
	volatile uint32_t DENALI_CTL_388;
	volatile uint32_t DENALI_CTL_389;
	volatile uint32_t DENALI_CTL_390;
	volatile uint32_t DENALI_CTL_391;
	volatile uint32_t DENALI_CTL_392;
	volatile uint32_t DENALI_CTL_393;
	volatile uint32_t DENALI_CTL_394;
	volatile uint32_t DENALI_CTL_395;
	volatile uint32_t DENALI_CTL_396;
	volatile uint32_t DENALI_CTL_397;
	volatile uint32_t DENALI_CTL_398;
	volatile uint32_t DENALI_CTL_399;
	volatile uint32_t DENALI_CTL_400;
	volatile uint32_t DENALI_CTL_401;
	volatile uint32_t DENALI_CTL_402;
	volatile uint32_t DENALI_CTL_403;
	volatile uint32_t DENALI_CTL_404;
	volatile uint32_t DENALI_CTL_405;
	volatile uint32_t DENALI_CTL_406;
	volatile uint32_t DENALI_CTL_407;
	volatile uint32_t DENALI_CTL_408;
	volatile uint32_t DENALI_CTL_409;
	volatile uint32_t DENALI_CTL_410;
	volatile uint32_t DENALI_CTL_411;
	volatile uint32_t DENALI_CTL_412;
	volatile uint32_t DENALI_CTL_413;
	volatile uint32_t DENALI_CTL_414;
	volatile uint32_t DENALI_CTL_415;
	volatile uint32_t DENALI_CTL_416;
	volatile uint32_t DENALI_CTL_417;
	volatile uint32_t DENALI_CTL_418;
	volatile uint32_t DENALI_CTL_419;
	volatile uint32_t DENALI_CTL_420;
	volatile uint32_t DENALI_CTL_421;
	volatile uint32_t DENALI_CTL_422;
	volatile uint32_t DENALI_CTL_423;
	volatile uint32_t DENALI_CTL_424;
	volatile uint32_t DENALI_CTL_425;
	volatile uint32_t DENALI_CTL_426;
	volatile uint32_t DENALI_CTL_427;
	volatile uint32_t DENALI_CTL_428;
	volatile uint32_t DENALI_CTL_429;
	volatile uint32_t DENALI_CTL_430;
	volatile uint32_t DENALI_CTL_431;
	volatile uint32_t DENALI_CTL_432;
	volatile uint32_t DENALI_CTL_433;
	volatile uint32_t DENALI_CTL_434;
	volatile uint32_t DENALI_CTL_435;
	volatile uint32_t DENALI_CTL_436;
	volatile uint32_t DENALI_CTL_437;
	volatile uint32_t DENALI_CTL_438;
	volatile uint32_t DENALI_CTL_439;
	volatile uint32_t DENALI_CTL_440;
	volatile uint32_t DENALI_CTL_441;
	volatile uint32_t DENALI_CTL_442;
	volatile uint32_t DENALI_CTL_443;
	volatile uint32_t DENALI_CTL_444;
	volatile uint32_t DENALI_CTL_445;
	volatile uint32_t DENALI_CTL_446;
	volatile uint32_t DENALI_CTL_447;
	volatile uint32_t DENALI_CTL_448;
	volatile uint32_t DENALI_CTL_449;
	volatile uint32_t DENALI_CTL_450;
	volatile uint32_t DENALI_CTL_451;
	volatile uint32_t DENALI_CTL_452;
	volatile uint32_t DENALI_CTL_453;
	volatile uint32_t DENALI_CTL_454;
	volatile uint32_t DENALI_CTL_455;
	volatile uint32_t DENALI_CTL_456;
	volatile uint32_t DENALI_CTL_457;
	volatile uint32_t DENALI_CTL_458;
	volatile char pad__0[0x18D4U];
	volatile uint32_t DENALI_PI_0;
	volatile uint32_t DENALI_PI_1;
	volatile uint32_t DENALI_PI_2;
	volatile uint32_t DENALI_PI_3;
	volatile uint32_t DENALI_PI_4;
	volatile uint32_t DENALI_PI_5;
	volatile uint32_t DENALI_PI_6;
	volatile uint32_t DENALI_PI_7;
	volatile uint32_t DENALI_PI_8;
	volatile uint32_t DENALI_PI_9;
	volatile uint32_t DENALI_PI_10;
	volatile uint32_t DENALI_PI_11;
	volatile uint32_t DENALI_PI_12;
	volatile uint32_t DENALI_PI_13;
	volatile uint32_t DENALI_PI_14;
	volatile uint32_t DENALI_PI_15;
	volatile uint32_t DENALI_PI_16;
	volatile uint32_t DENALI_PI_17;
	volatile uint32_t DENALI_PI_18;
	volatile uint32_t DENALI_PI_19;
	volatile uint32_t DENALI_PI_20;
	volatile uint32_t DENALI_PI_21;
	volatile uint32_t DENALI_PI_22;
	volatile uint32_t DENALI_PI_23;
	volatile uint32_t DENALI_PI_24;
	volatile uint32_t DENALI_PI_25;
	volatile uint32_t DENALI_PI_26;
	volatile uint32_t DENALI_PI_27;
	volatile uint32_t DENALI_PI_28;
	volatile uint32_t DENALI_PI_29;
	volatile uint32_t DENALI_PI_30;
	volatile uint32_t DENALI_PI_31;
	volatile uint32_t DENALI_PI_32;
	volatile uint32_t DENALI_PI_33;
	volatile uint32_t DENALI_PI_34;
	volatile uint32_t DENALI_PI_35;
	volatile uint32_t DENALI_PI_36;
	volatile uint32_t DENALI_PI_37;
	volatile uint32_t DENALI_PI_38;
	volatile uint32_t DENALI_PI_39;
	volatile uint32_t DENALI_PI_40;
	volatile uint32_t DENALI_PI_41;
	volatile uint32_t DENALI_PI_42;
	volatile uint32_t DENALI_PI_43;
	volatile uint32_t DENALI_PI_44;
	volatile uint32_t DENALI_PI_45;
	volatile uint32_t DENALI_PI_46;
	volatile uint32_t DENALI_PI_47;
	volatile uint32_t DENALI_PI_48;
	volatile uint32_t DENALI_PI_49;
	volatile uint32_t DENALI_PI_50;
	volatile uint32_t DENALI_PI_51;
	volatile uint32_t DENALI_PI_52;
	volatile uint32_t DENALI_PI_53;
	volatile uint32_t DENALI_PI_54;
	volatile uint32_t DENALI_PI_55;
	volatile uint32_t DENALI_PI_56;
	volatile uint32_t DENALI_PI_57;
	volatile uint32_t DENALI_PI_58;
	volatile uint32_t DENALI_PI_59;
	volatile uint32_t DENALI_PI_60;
	volatile uint32_t DENALI_PI_61;
	volatile uint32_t DENALI_PI_62;
	volatile uint32_t DENALI_PI_63;
	volatile uint32_t DENALI_PI_64;
	volatile uint32_t DENALI_PI_65;
	volatile uint32_t DENALI_PI_66;
	volatile uint32_t DENALI_PI_67;
	volatile uint32_t DENALI_PI_68;
	volatile uint32_t DENALI_PI_69;
	volatile uint32_t DENALI_PI_70;
	volatile uint32_t DENALI_PI_71;
	volatile uint32_t DENALI_PI_72;
	volatile uint32_t DENALI_PI_73;
	volatile uint32_t DENALI_PI_74;
	volatile uint32_t DENALI_PI_75;
	volatile uint32_t DENALI_PI_76;
	volatile uint32_t DENALI_PI_77;
	volatile uint32_t DENALI_PI_78;
	volatile uint32_t DENALI_PI_79;
	volatile uint32_t DENALI_PI_80;
	volatile uint32_t DENALI_PI_81;
	volatile uint32_t DENALI_PI_82;
	volatile uint32_t DENALI_PI_83;
	volatile uint32_t DENALI_PI_84;
	volatile uint32_t DENALI_PI_85;
	volatile uint32_t DENALI_PI_86;
	volatile uint32_t DENALI_PI_87;
	volatile uint32_t DENALI_PI_88;
	volatile uint32_t DENALI_PI_89;
	volatile uint32_t DENALI_PI_90;
	volatile uint32_t DENALI_PI_91;
	volatile uint32_t DENALI_PI_92;
	volatile uint32_t DENALI_PI_93;
	volatile uint32_t DENALI_PI_94;
	volatile uint32_t DENALI_PI_95;
	volatile uint32_t DENALI_PI_96;
	volatile uint32_t DENALI_PI_97;
	volatile uint32_t DENALI_PI_98;
	volatile uint32_t DENALI_PI_99;
	volatile uint32_t DENALI_PI_100;
	volatile uint32_t DENALI_PI_101;
	volatile uint32_t DENALI_PI_102;
	volatile uint32_t DENALI_PI_103;
	volatile uint32_t DENALI_PI_104;
	volatile uint32_t DENALI_PI_105;
	volatile uint32_t DENALI_PI_106;
	volatile uint32_t DENALI_PI_107;
	volatile uint32_t DENALI_PI_108;
	volatile uint32_t DENALI_PI_109;
	volatile uint32_t DENALI_PI_110;
	volatile uint32_t DENALI_PI_111;
	volatile uint32_t DENALI_PI_112;
	volatile uint32_t DENALI_PI_113;
	volatile uint32_t DENALI_PI_114;
	volatile uint32_t DENALI_PI_115;
	volatile uint32_t DENALI_PI_116;
	volatile uint32_t DENALI_PI_117;
	volatile uint32_t DENALI_PI_118;
	volatile uint32_t DENALI_PI_119;
	volatile uint32_t DENALI_PI_120;
	volatile uint32_t DENALI_PI_121;
	volatile uint32_t DENALI_PI_122;
	volatile uint32_t DENALI_PI_123;
	volatile uint32_t DENALI_PI_124;
	volatile uint32_t DENALI_PI_125;
	volatile uint32_t DENALI_PI_126;
	volatile uint32_t DENALI_PI_127;
	volatile uint32_t DENALI_PI_128;
	volatile uint32_t DENALI_PI_129;
	volatile uint32_t DENALI_PI_130;
	volatile uint32_t DENALI_PI_131;
	volatile uint32_t DENALI_PI_132;
	volatile uint32_t DENALI_PI_133;
	volatile uint32_t DENALI_PI_134;
	volatile uint32_t DENALI_PI_135;
	volatile uint32_t DENALI_PI_136;
	volatile uint32_t DENALI_PI_137;
	volatile uint32_t DENALI_PI_138;
	volatile uint32_t DENALI_PI_139;
	volatile uint32_t DENALI_PI_140;
	volatile uint32_t DENALI_PI_141;
	volatile uint32_t DENALI_PI_142;
	volatile uint32_t DENALI_PI_143;
	volatile uint32_t DENALI_PI_144;
	volatile uint32_t DENALI_PI_145;
	volatile uint32_t DENALI_PI_146;
	volatile uint32_t DENALI_PI_147;
	volatile uint32_t DENALI_PI_148;
	volatile uint32_t DENALI_PI_149;
	volatile uint32_t DENALI_PI_150;
	volatile uint32_t DENALI_PI_151;
	volatile uint32_t DENALI_PI_152;
	volatile uint32_t DENALI_PI_153;
	volatile uint32_t DENALI_PI_154;
	volatile uint32_t DENALI_PI_155;
	volatile uint32_t DENALI_PI_156;
	volatile uint32_t DENALI_PI_157;
	volatile uint32_t DENALI_PI_158;
	volatile uint32_t DENALI_PI_159;
	volatile uint32_t DENALI_PI_160;
	volatile uint32_t DENALI_PI_161;
	volatile uint32_t DENALI_PI_162;
	volatile uint32_t DENALI_PI_163;
	volatile uint32_t DENALI_PI_164;
	volatile uint32_t DENALI_PI_165;
	volatile uint32_t DENALI_PI_166;
	volatile uint32_t DENALI_PI_167;
	volatile uint32_t DENALI_PI_168;
	volatile uint32_t DENALI_PI_169;
	volatile uint32_t DENALI_PI_170;
	volatile uint32_t DENALI_PI_171;
	volatile uint32_t DENALI_PI_172;
	volatile uint32_t DENALI_PI_173;
	volatile uint32_t DENALI_PI_174;
	volatile uint32_t DENALI_PI_175;
	volatile uint32_t DENALI_PI_176;
	volatile uint32_t DENALI_PI_177;
	volatile uint32_t DENALI_PI_178;
	volatile uint32_t DENALI_PI_179;
	volatile uint32_t DENALI_PI_180;
	volatile uint32_t DENALI_PI_181;
	volatile uint32_t DENALI_PI_182;
	volatile uint32_t DENALI_PI_183;
	volatile uint32_t DENALI_PI_184;
	volatile uint32_t DENALI_PI_185;
	volatile uint32_t DENALI_PI_186;
	volatile uint32_t DENALI_PI_187;
	volatile uint32_t DENALI_PI_188;
	volatile uint32_t DENALI_PI_189;
	volatile uint32_t DENALI_PI_190;
	volatile uint32_t DENALI_PI_191;
	volatile uint32_t DENALI_PI_192;
	volatile uint32_t DENALI_PI_193;
	volatile uint32_t DENALI_PI_194;
	volatile uint32_t DENALI_PI_195;
	volatile uint32_t DENALI_PI_196;
	volatile uint32_t DENALI_PI_197;
	volatile uint32_t DENALI_PI_198;
	volatile uint32_t DENALI_PI_199;
	volatile uint32_t DENALI_PI_200;
	volatile uint32_t DENALI_PI_201;
	volatile uint32_t DENALI_PI_202;
	volatile uint32_t DENALI_PI_203;
	volatile uint32_t DENALI_PI_204;
	volatile uint32_t DENALI_PI_205;
	volatile uint32_t DENALI_PI_206;
	volatile uint32_t DENALI_PI_207;
	volatile uint32_t DENALI_PI_208;
	volatile uint32_t DENALI_PI_209;
	volatile uint32_t DENALI_PI_210;
	volatile uint32_t DENALI_PI_211;
	volatile uint32_t DENALI_PI_212;
	volatile uint32_t DENALI_PI_213;
	volatile uint32_t DENALI_PI_214;
	volatile uint32_t DENALI_PI_215;
	volatile uint32_t DENALI_PI_216;
	volatile uint32_t DENALI_PI_217;
	volatile uint32_t DENALI_PI_218;
	volatile uint32_t DENALI_PI_219;
	volatile uint32_t DENALI_PI_220;
	volatile uint32_t DENALI_PI_221;
	volatile uint32_t DENALI_PI_222;
	volatile uint32_t DENALI_PI_223;
	volatile uint32_t DENALI_PI_224;
	volatile uint32_t DENALI_PI_225;
	volatile uint32_t DENALI_PI_226;
	volatile uint32_t DENALI_PI_227;
	volatile uint32_t DENALI_PI_228;
	volatile uint32_t DENALI_PI_229;
	volatile uint32_t DENALI_PI_230;
	volatile uint32_t DENALI_PI_231;
	volatile uint32_t DENALI_PI_232;
	volatile uint32_t DENALI_PI_233;
	volatile uint32_t DENALI_PI_234;
	volatile uint32_t DENALI_PI_235;
	volatile uint32_t DENALI_PI_236;
	volatile uint32_t DENALI_PI_237;
	volatile uint32_t DENALI_PI_238;
	volatile uint32_t DENALI_PI_239;
	volatile uint32_t DENALI_PI_240;
	volatile uint32_t DENALI_PI_241;
	volatile uint32_t DENALI_PI_242;
	volatile uint32_t DENALI_PI_243;
	volatile uint32_t DENALI_PI_244;
	volatile uint32_t DENALI_PI_245;
	volatile uint32_t DENALI_PI_246;
	volatile uint32_t DENALI_PI_247;
	volatile uint32_t DENALI_PI_248;
	volatile uint32_t DENALI_PI_249;
	volatile uint32_t DENALI_PI_250;
	volatile uint32_t DENALI_PI_251;
	volatile uint32_t DENALI_PI_252;
	volatile uint32_t DENALI_PI_253;
	volatile uint32_t DENALI_PI_254;
	volatile uint32_t DENALI_PI_255;
	volatile uint32_t DENALI_PI_256;
	volatile uint32_t DENALI_PI_257;
	volatile uint32_t DENALI_PI_258;
	volatile uint32_t DENALI_PI_259;
	volatile uint32_t DENALI_PI_260;
	volatile uint32_t DENALI_PI_261;
	volatile uint32_t DENALI_PI_262;
	volatile uint32_t DENALI_PI_263;
	volatile uint32_t DENALI_PI_264;
	volatile uint32_t DENALI_PI_265;
	volatile uint32_t DENALI_PI_266;
	volatile uint32_t DENALI_PI_267;
	volatile uint32_t DENALI_PI_268;
	volatile uint32_t DENALI_PI_269;
	volatile uint32_t DENALI_PI_270;
	volatile uint32_t DENALI_PI_271;
	volatile uint32_t DENALI_PI_272;
	volatile uint32_t DENALI_PI_273;
	volatile uint32_t DENALI_PI_274;
	volatile uint32_t DENALI_PI_275;
	volatile uint32_t DENALI_PI_276;
	volatile uint32_t DENALI_PI_277;
	volatile uint32_t DENALI_PI_278;
	volatile uint32_t DENALI_PI_279;
	volatile uint32_t DENALI_PI_280;
	volatile uint32_t DENALI_PI_281;
	volatile uint32_t DENALI_PI_282;
	volatile uint32_t DENALI_PI_283;
	volatile uint32_t DENALI_PI_284;
	volatile uint32_t DENALI_PI_285;
	volatile uint32_t DENALI_PI_286;
	volatile uint32_t DENALI_PI_287;
	volatile uint32_t DENALI_PI_288;
	volatile uint32_t DENALI_PI_289;
	volatile uint32_t DENALI_PI_290;
	volatile uint32_t DENALI_PI_291;
	volatile uint32_t DENALI_PI_292;
	volatile uint32_t DENALI_PI_293;
	volatile uint32_t DENALI_PI_294;
	volatile uint32_t DENALI_PI_295;
	volatile uint32_t DENALI_PI_296;
	volatile uint32_t DENALI_PI_297;
	volatile uint32_t DENALI_PI_298;
	volatile uint32_t DENALI_PI_299;
	volatile char pad__1[0x1B50U];
	volatile uint32_t DENALI_PHY_0;
	volatile uint32_t DENALI_PHY_1;
	volatile uint32_t DENALI_PHY_2;
	volatile uint32_t DENALI_PHY_3;
	volatile uint32_t DENALI_PHY_4;
	volatile uint32_t DENALI_PHY_5;
	volatile uint32_t DENALI_PHY_6;
	volatile uint32_t DENALI_PHY_7;
	volatile uint32_t DENALI_PHY_8;
	volatile uint32_t DENALI_PHY_9;
	volatile uint32_t DENALI_PHY_10;
	volatile uint32_t DENALI_PHY_11;
	volatile uint32_t DENALI_PHY_12;
	volatile uint32_t DENALI_PHY_13;
	volatile uint32_t DENALI_PHY_14;
	volatile uint32_t DENALI_PHY_15;
	volatile uint32_t DENALI_PHY_16;
	volatile uint32_t DENALI_PHY_17;
	volatile uint32_t DENALI_PHY_18;
	volatile uint32_t DENALI_PHY_19;
	volatile uint32_t DENALI_PHY_20;
	volatile uint32_t DENALI_PHY_21;
	volatile uint32_t DENALI_PHY_22;
	volatile uint32_t DENALI_PHY_23;
	volatile uint32_t DENALI_PHY_24;
	volatile uint32_t DENALI_PHY_25;
	volatile uint32_t DENALI_PHY_26;
	volatile uint32_t DENALI_PHY_27;
	volatile uint32_t DENALI_PHY_28;
	volatile uint32_t DENALI_PHY_29;
	volatile uint32_t DENALI_PHY_30;
	volatile uint32_t DENALI_PHY_31;
	volatile uint32_t DENALI_PHY_32;
	volatile uint32_t DENALI_PHY_33;
	volatile uint32_t DENALI_PHY_34;
	volatile uint32_t DENALI_PHY_35;
	volatile uint32_t DENALI_PHY_36;
	volatile uint32_t DENALI_PHY_37;
	volatile uint32_t DENALI_PHY_38;
	volatile uint32_t DENALI_PHY_39;
	volatile uint32_t DENALI_PHY_40;
	volatile uint32_t DENALI_PHY_41;
	volatile uint32_t DENALI_PHY_42;
	volatile uint32_t DENALI_PHY_43;
	volatile uint32_t DENALI_PHY_44;
	volatile uint32_t DENALI_PHY_45;
	volatile uint32_t DENALI_PHY_46;
	volatile uint32_t DENALI_PHY_47;
	volatile uint32_t DENALI_PHY_48;
	volatile uint32_t DENALI_PHY_49;
	volatile uint32_t DENALI_PHY_50;
	volatile uint32_t DENALI_PHY_51;
	volatile uint32_t DENALI_PHY_52;
	volatile uint32_t DENALI_PHY_53;
	volatile uint32_t DENALI_PHY_54;
	volatile uint32_t DENALI_PHY_55;
	volatile uint32_t DENALI_PHY_56;
	volatile uint32_t DENALI_PHY_57;
	volatile uint32_t DENALI_PHY_58;
	volatile uint32_t DENALI_PHY_59;
	volatile uint32_t DENALI_PHY_60;
	volatile uint32_t DENALI_PHY_61;
	volatile uint32_t DENALI_PHY_62;
	volatile uint32_t DENALI_PHY_63;
	volatile uint32_t DENALI_PHY_64;
	volatile uint32_t DENALI_PHY_65;
	volatile uint32_t DENALI_PHY_66;
	volatile uint32_t DENALI_PHY_67;
	volatile uint32_t DENALI_PHY_68;
	volatile uint32_t DENALI_PHY_69;
	volatile uint32_t DENALI_PHY_70;
	volatile uint32_t DENALI_PHY_71;
	volatile uint32_t DENALI_PHY_72;
	volatile uint32_t DENALI_PHY_73;
	volatile uint32_t DENALI_PHY_74;
	volatile uint32_t DENALI_PHY_75;
	volatile uint32_t DENALI_PHY_76;
	volatile uint32_t DENALI_PHY_77;
	volatile uint32_t DENALI_PHY_78;
	volatile uint32_t DENALI_PHY_79;
	volatile uint32_t DENALI_PHY_80;
	volatile uint32_t DENALI_PHY_81;
	volatile uint32_t DENALI_PHY_82;
	volatile uint32_t DENALI_PHY_83;
	volatile uint32_t DENALI_PHY_84;
	volatile uint32_t DENALI_PHY_85;
	volatile uint32_t DENALI_PHY_86;
	volatile uint32_t DENALI_PHY_87;
	volatile uint32_t DENALI_PHY_88;
	volatile uint32_t DENALI_PHY_89;
	volatile uint32_t DENALI_PHY_90;
	volatile uint32_t DENALI_PHY_91;
	volatile uint32_t DENALI_PHY_92;
	volatile uint32_t DENALI_PHY_93;
	volatile uint32_t DENALI_PHY_94;
	volatile uint32_t DENALI_PHY_95;
	volatile uint32_t DENALI_PHY_96;
	volatile uint32_t DENALI_PHY_97;
	volatile uint32_t DENALI_PHY_98;
	volatile uint32_t DENALI_PHY_99;
	volatile uint32_t DENALI_PHY_100;
	volatile uint32_t DENALI_PHY_101;
	volatile uint32_t DENALI_PHY_102;
	volatile uint32_t DENALI_PHY_103;
	volatile uint32_t DENALI_PHY_104;
	volatile uint32_t DENALI_PHY_105;
	volatile uint32_t DENALI_PHY_106;
	volatile uint32_t DENALI_PHY_107;
	volatile uint32_t DENALI_PHY_108;
	volatile uint32_t DENALI_PHY_109;
	volatile uint32_t DENALI_PHY_110;
	volatile uint32_t DENALI_PHY_111;
	volatile uint32_t DENALI_PHY_112;
	volatile uint32_t DENALI_PHY_113;
	volatile uint32_t DENALI_PHY_114;
	volatile uint32_t DENALI_PHY_115;
	volatile uint32_t DENALI_PHY_116;
	volatile uint32_t DENALI_PHY_117;
	volatile uint32_t DENALI_PHY_118;
	volatile uint32_t DENALI_PHY_119;
	volatile uint32_t DENALI_PHY_120;
	volatile uint32_t DENALI_PHY_121;
	volatile uint32_t DENALI_PHY_122;
	volatile uint32_t DENALI_PHY_123;
	volatile uint32_t DENALI_PHY_124;
	volatile uint32_t DENALI_PHY_125;
	volatile uint32_t DENALI_PHY_126;
	volatile uint32_t DENALI_PHY_127;
	volatile uint32_t DENALI_PHY_128;
	volatile uint32_t DENALI_PHY_129;
	volatile uint32_t DENALI_PHY_130;
	volatile uint32_t DENALI_PHY_131;
	volatile uint32_t DENALI_PHY_132;
	volatile uint32_t DENALI_PHY_133;
	volatile uint32_t DENALI_PHY_134;
	volatile uint32_t DENALI_PHY_135;
	volatile uint32_t DENALI_PHY_136;
	volatile uint32_t DENALI_PHY_137;
	volatile uint32_t DENALI_PHY_138;
	volatile uint32_t DENALI_PHY_139;
	volatile char pad__2[0x1D0U];
	volatile uint32_t DENALI_PHY_256;
	volatile uint32_t DENALI_PHY_257;
	volatile uint32_t DENALI_PHY_258;
	volatile uint32_t DENALI_PHY_259;
	volatile uint32_t DENALI_PHY_260;
	volatile uint32_t DENALI_PHY_261;
	volatile uint32_t DENALI_PHY_262;
	volatile uint32_t DENALI_PHY_263;
	volatile uint32_t DENALI_PHY_264;
	volatile uint32_t DENALI_PHY_265;
	volatile uint32_t DENALI_PHY_266;
	volatile uint32_t DENALI_PHY_267;
	volatile uint32_t DENALI_PHY_268;
	volatile uint32_t DENALI_PHY_269;
	volatile uint32_t DENALI_PHY_270;
	volatile uint32_t DENALI_PHY_271;
	volatile uint32_t DENALI_PHY_272;
	volatile uint32_t DENALI_PHY_273;
	volatile uint32_t DENALI_PHY_274;
	volatile uint32_t DENALI_PHY_275;
	volatile uint32_t DENALI_PHY_276;
	volatile uint32_t DENALI_PHY_277;
	volatile uint32_t DENALI_PHY_278;
	volatile uint32_t DENALI_PHY_279;
	volatile uint32_t DENALI_PHY_280;
	volatile uint32_t DENALI_PHY_281;
	volatile uint32_t DENALI_PHY_282;
	volatile uint32_t DENALI_PHY_283;
	volatile uint32_t DENALI_PHY_284;
	volatile uint32_t DENALI_PHY_285;
	volatile uint32_t DENALI_PHY_286;
	volatile uint32_t DENALI_PHY_287;
	volatile uint32_t DENALI_PHY_288;
	volatile uint32_t DENALI_PHY_289;
	volatile uint32_t DENALI_PHY_290;
	volatile uint32_t DENALI_PHY_291;
	volatile uint32_t DENALI_PHY_292;
	volatile uint32_t DENALI_PHY_293;
	volatile uint32_t DENALI_PHY_294;
	volatile uint32_t DENALI_PHY_295;
	volatile uint32_t DENALI_PHY_296;
	volatile uint32_t DENALI_PHY_297;
	volatile uint32_t DENALI_PHY_298;
	volatile uint32_t DENALI_PHY_299;
	volatile uint32_t DENALI_PHY_300;
	volatile uint32_t DENALI_PHY_301;
	volatile uint32_t DENALI_PHY_302;
	volatile uint32_t DENALI_PHY_303;
	volatile uint32_t DENALI_PHY_304;
	volatile uint32_t DENALI_PHY_305;
	volatile uint32_t DENALI_PHY_306;
	volatile uint32_t DENALI_PHY_307;
	volatile uint32_t DENALI_PHY_308;
	volatile uint32_t DENALI_PHY_309;
	volatile uint32_t DENALI_PHY_310;
	volatile uint32_t DENALI_PHY_311;
	volatile uint32_t DENALI_PHY_312;
	volatile uint32_t DENALI_PHY_313;
	volatile uint32_t DENALI_PHY_314;
	volatile uint32_t DENALI_PHY_315;
	volatile uint32_t DENALI_PHY_316;
	volatile uint32_t DENALI_PHY_317;
	volatile uint32_t DENALI_PHY_318;
	volatile uint32_t DENALI_PHY_319;
	volatile uint32_t DENALI_PHY_320;
	volatile uint32_t DENALI_PHY_321;
	volatile uint32_t DENALI_PHY_322;
	volatile uint32_t DENALI_PHY_323;
	volatile uint32_t DENALI_PHY_324;
	volatile uint32_t DENALI_PHY_325;
	volatile uint32_t DENALI_PHY_326;
	volatile uint32_t DENALI_PHY_327;
	volatile uint32_t DENALI_PHY_328;
	volatile uint32_t DENALI_PHY_329;
	volatile uint32_t DENALI_PHY_330;
	volatile uint32_t DENALI_PHY_331;
	volatile uint32_t DENALI_PHY_332;
	volatile uint32_t DENALI_PHY_333;
	volatile uint32_t DENALI_PHY_334;
	volatile uint32_t DENALI_PHY_335;
	volatile uint32_t DENALI_PHY_336;
	volatile uint32_t DENALI_PHY_337;
	volatile uint32_t DENALI_PHY_338;
	volatile uint32_t DENALI_PHY_339;
	volatile uint32_t DENALI_PHY_340;
	volatile uint32_t DENALI_PHY_341;
	volatile uint32_t DENALI_PHY_342;
	volatile uint32_t DENALI_PHY_343;
	volatile uint32_t DENALI_PHY_344;
	volatile uint32_t DENALI_PHY_345;
	volatile uint32_t DENALI_PHY_346;
	volatile uint32_t DENALI_PHY_347;
	volatile uint32_t DENALI_PHY_348;
	volatile uint32_t DENALI_PHY_349;
	volatile uint32_t DENALI_PHY_350;
	volatile uint32_t DENALI_PHY_351;
	volatile uint32_t DENALI_PHY_352;
	volatile uint32_t DENALI_PHY_353;
	volatile uint32_t DENALI_PHY_354;
	volatile uint32_t DENALI_PHY_355;
	volatile uint32_t DENALI_PHY_356;
	volatile uint32_t DENALI_PHY_357;
	volatile uint32_t DENALI_PHY_358;
	volatile uint32_t DENALI_PHY_359;
	volatile uint32_t DENALI_PHY_360;
	volatile uint32_t DENALI_PHY_361;
	volatile uint32_t DENALI_PHY_362;
	volatile uint32_t DENALI_PHY_363;
	volatile uint32_t DENALI_PHY_364;
	volatile uint32_t DENALI_PHY_365;
	volatile uint32_t DENALI_PHY_366;
	volatile uint32_t DENALI_PHY_367;
	volatile uint32_t DENALI_PHY_368;
	volatile uint32_t DENALI_PHY_369;
	volatile uint32_t DENALI_PHY_370;
	volatile uint32_t DENALI_PHY_371;
	volatile uint32_t DENALI_PHY_372;
	volatile uint32_t DENALI_PHY_373;
	volatile uint32_t DENALI_PHY_374;
	volatile uint32_t DENALI_PHY_375;
	volatile uint32_t DENALI_PHY_376;
	volatile uint32_t DENALI_PHY_377;
	volatile uint32_t DENALI_PHY_378;
	volatile uint32_t DENALI_PHY_379;
	volatile uint32_t DENALI_PHY_380;
	volatile uint32_t DENALI_PHY_381;
	volatile uint32_t DENALI_PHY_382;
	volatile uint32_t DENALI_PHY_383;
	volatile uint32_t DENALI_PHY_384;
	volatile uint32_t DENALI_PHY_385;
	volatile uint32_t DENALI_PHY_386;
	volatile uint32_t DENALI_PHY_387;
	volatile uint32_t DENALI_PHY_388;
	volatile uint32_t DENALI_PHY_389;
	volatile uint32_t DENALI_PHY_390;
	volatile uint32_t DENALI_PHY_391;
	volatile uint32_t DENALI_PHY_392;
	volatile uint32_t DENALI_PHY_393;
	volatile uint32_t DENALI_PHY_394;
	volatile uint32_t DENALI_PHY_395;
	volatile char pad__3[0x1D0U];
	volatile uint32_t DENALI_PHY_512;
	volatile uint32_t DENALI_PHY_513;
	volatile uint32_t DENALI_PHY_514;
	volatile uint32_t DENALI_PHY_515;
	volatile uint32_t DENALI_PHY_516;
	volatile uint32_t DENALI_PHY_517;
	volatile uint32_t DENALI_PHY_518;
	volatile uint32_t DENALI_PHY_519;
	volatile uint32_t DENALI_PHY_520;
	volatile uint32_t DENALI_PHY_521;
	volatile uint32_t DENALI_PHY_522;
	volatile uint32_t DENALI_PHY_523;
	volatile uint32_t DENALI_PHY_524;
	volatile uint32_t DENALI_PHY_525;
	volatile uint32_t DENALI_PHY_526;
	volatile uint32_t DENALI_PHY_527;
	volatile uint32_t DENALI_PHY_528;
	volatile uint32_t DENALI_PHY_529;
	volatile uint32_t DENALI_PHY_530;
	volatile uint32_t DENALI_PHY_531;
	volatile uint32_t DENALI_PHY_532;
	volatile uint32_t DENALI_PHY_533;
	volatile uint32_t DENALI_PHY_534;
	volatile uint32_t DENALI_PHY_535;
	volatile uint32_t DENALI_PHY_536;
	volatile uint32_t DENALI_PHY_537;
	volatile uint32_t DENALI_PHY_538;
	volatile uint32_t DENALI_PHY_539;
	volatile uint32_t DENALI_PHY_540;
	volatile uint32_t DENALI_PHY_541;
	volatile uint32_t DENALI_PHY_542;
	volatile uint32_t DENALI_PHY_543;
	volatile uint32_t DENALI_PHY_544;
	volatile uint32_t DENALI_PHY_545;
	volatile uint32_t DENALI_PHY_546;
	volatile uint32_t DENALI_PHY_547;
	volatile uint32_t DENALI_PHY_548;
	volatile uint32_t DENALI_PHY_549;
	volatile uint32_t DENALI_PHY_550;
	volatile uint32_t DENALI_PHY_551;
	volatile uint32_t DENALI_PHY_552;
	volatile uint32_t DENALI_PHY_553;
	volatile uint32_t DENALI_PHY_554;
	volatile uint32_t DENALI_PHY_555;
	volatile uint32_t DENALI_PHY_556;
	volatile uint32_t DENALI_PHY_557;
	volatile uint32_t DENALI_PHY_558;
	volatile uint32_t DENALI_PHY_559;
	volatile uint32_t DENALI_PHY_560;
	volatile uint32_t DENALI_PHY_561;
	volatile uint32_t DENALI_PHY_562;
	volatile uint32_t DENALI_PHY_563;
	volatile uint32_t DENALI_PHY_564;
	volatile uint32_t DENALI_PHY_565;
	volatile uint32_t DENALI_PHY_566;
	volatile uint32_t DENALI_PHY_567;
	volatile uint32_t DENALI_PHY_568;
	volatile uint32_t DENALI_PHY_569;
	volatile uint32_t DENALI_PHY_570;
	volatile uint32_t DENALI_PHY_571;
	volatile uint32_t DENALI_PHY_572;
	volatile uint32_t DENALI_PHY_573;
	volatile uint32_t DENALI_PHY_574;
	volatile uint32_t DENALI_PHY_575;
	volatile uint32_t DENALI_PHY_576;
	volatile uint32_t DENALI_PHY_577;
	volatile uint32_t DENALI_PHY_578;
	volatile uint32_t DENALI_PHY_579;
	volatile uint32_t DENALI_PHY_580;
	volatile uint32_t DENALI_PHY_581;
	volatile uint32_t DENALI_PHY_582;
	volatile uint32_t DENALI_PHY_583;
	volatile uint32_t DENALI_PHY_584;
	volatile uint32_t DENALI_PHY_585;
	volatile uint32_t DENALI_PHY_586;
	volatile uint32_t DENALI_PHY_587;
	volatile uint32_t DENALI_PHY_588;
	volatile uint32_t DENALI_PHY_589;
	volatile uint32_t DENALI_PHY_590;
	volatile uint32_t DENALI_PHY_591;
	volatile uint32_t DENALI_PHY_592;
	volatile uint32_t DENALI_PHY_593;
	volatile uint32_t DENALI_PHY_594;
	volatile uint32_t DENALI_PHY_595;
	volatile uint32_t DENALI_PHY_596;
	volatile uint32_t DENALI_PHY_597;
	volatile uint32_t DENALI_PHY_598;
	volatile uint32_t DENALI_PHY_599;
	volatile uint32_t DENALI_PHY_600;
	volatile uint32_t DENALI_PHY_601;
	volatile uint32_t DENALI_PHY_602;
	volatile uint32_t DENALI_PHY_603;
	volatile uint32_t DENALI_PHY_604;
	volatile uint32_t DENALI_PHY_605;
	volatile uint32_t DENALI_PHY_606;
	volatile uint32_t DENALI_PHY_607;
	volatile uint32_t DENALI_PHY_608;
	volatile uint32_t DENALI_PHY_609;
	volatile uint32_t DENALI_PHY_610;
	volatile uint32_t DENALI_PHY_611;
	volatile uint32_t DENALI_PHY_612;
	volatile uint32_t DENALI_PHY_613;
	volatile uint32_t DENALI_PHY_614;
	volatile uint32_t DENALI_PHY_615;
	volatile uint32_t DENALI_PHY_616;
	volatile uint32_t DENALI_PHY_617;
	volatile uint32_t DENALI_PHY_618;
	volatile uint32_t DENALI_PHY_619;
	volatile uint32_t DENALI_PHY_620;
	volatile uint32_t DENALI_PHY_621;
	volatile uint32_t DENALI_PHY_622;
	volatile uint32_t DENALI_PHY_623;
	volatile uint32_t DENALI_PHY_624;
	volatile uint32_t DENALI_PHY_625;
	volatile uint32_t DENALI_PHY_626;
	volatile uint32_t DENALI_PHY_627;
	volatile uint32_t DENALI_PHY_628;
	volatile uint32_t DENALI_PHY_629;
	volatile uint32_t DENALI_PHY_630;
	volatile uint32_t DENALI_PHY_631;
	volatile uint32_t DENALI_PHY_632;
	volatile uint32_t DENALI_PHY_633;
	volatile uint32_t DENALI_PHY_634;
	volatile uint32_t DENALI_PHY_635;
	volatile uint32_t DENALI_PHY_636;
	volatile uint32_t DENALI_PHY_637;
	volatile uint32_t DENALI_PHY_638;
	volatile uint32_t DENALI_PHY_639;
	volatile uint32_t DENALI_PHY_640;
	volatile uint32_t DENALI_PHY_641;
	volatile uint32_t DENALI_PHY_642;
	volatile uint32_t DENALI_PHY_643;
	volatile uint32_t DENALI_PHY_644;
	volatile uint32_t DENALI_PHY_645;
	volatile uint32_t DENALI_PHY_646;
	volatile uint32_t DENALI_PHY_647;
	volatile uint32_t DENALI_PHY_648;
	volatile uint32_t DENALI_PHY_649;
	volatile uint32_t DENALI_PHY_650;
	volatile uint32_t DENALI_PHY_651;
	volatile char pad__4[0x1D0U];
	volatile uint32_t DENALI_PHY_768;
	volatile uint32_t DENALI_PHY_769;
	volatile uint32_t DENALI_PHY_770;
	volatile uint32_t DENALI_PHY_771;
	volatile uint32_t DENALI_PHY_772;
	volatile uint32_t DENALI_PHY_773;
	volatile uint32_t DENALI_PHY_774;
	volatile uint32_t DENALI_PHY_775;
	volatile uint32_t DENALI_PHY_776;
	volatile uint32_t DENALI_PHY_777;
	volatile uint32_t DENALI_PHY_778;
	volatile uint32_t DENALI_PHY_779;
	volatile uint32_t DENALI_PHY_780;
	volatile uint32_t DENALI_PHY_781;
	volatile uint32_t DENALI_PHY_782;
	volatile uint32_t DENALI_PHY_783;
	volatile uint32_t DENALI_PHY_784;
	volatile uint32_t DENALI_PHY_785;
	volatile uint32_t DENALI_PHY_786;
	volatile uint32_t DENALI_PHY_787;
	volatile uint32_t DENALI_PHY_788;
	volatile uint32_t DENALI_PHY_789;
	volatile uint32_t DENALI_PHY_790;
	volatile uint32_t DENALI_PHY_791;
	volatile uint32_t DENALI_PHY_792;
	volatile uint32_t DENALI_PHY_793;
	volatile uint32_t DENALI_PHY_794;
	volatile uint32_t DENALI_PHY_795;
	volatile uint32_t DENALI_PHY_796;
	volatile uint32_t DENALI_PHY_797;
	volatile uint32_t DENALI_PHY_798;
	volatile uint32_t DENALI_PHY_799;
	volatile uint32_t DENALI_PHY_800;
	volatile uint32_t DENALI_PHY_801;
	volatile uint32_t DENALI_PHY_802;
	volatile uint32_t DENALI_PHY_803;
	volatile uint32_t DENALI_PHY_804;
	volatile uint32_t DENALI_PHY_805;
	volatile uint32_t DENALI_PHY_806;
	volatile uint32_t DENALI_PHY_807;
	volatile uint32_t DENALI_PHY_808;
	volatile uint32_t DENALI_PHY_809;
	volatile uint32_t DENALI_PHY_810;
	volatile uint32_t DENALI_PHY_811;
	volatile uint32_t DENALI_PHY_812;
	volatile uint32_t DENALI_PHY_813;
	volatile uint32_t DENALI_PHY_814;
	volatile uint32_t DENALI_PHY_815;
	volatile uint32_t DENALI_PHY_816;
	volatile uint32_t DENALI_PHY_817;
	volatile uint32_t DENALI_PHY_818;
	volatile uint32_t DENALI_PHY_819;
	volatile uint32_t DENALI_PHY_820;
	volatile uint32_t DENALI_PHY_821;
	volatile uint32_t DENALI_PHY_822;
	volatile uint32_t DENALI_PHY_823;
	volatile uint32_t DENALI_PHY_824;
	volatile uint32_t DENALI_PHY_825;
	volatile uint32_t DENALI_PHY_826;
	volatile uint32_t DENALI_PHY_827;
	volatile uint32_t DENALI_PHY_828;
	volatile uint32_t DENALI_PHY_829;
	volatile uint32_t DENALI_PHY_830;
	volatile uint32_t DENALI_PHY_831;
	volatile uint32_t DENALI_PHY_832;
	volatile uint32_t DENALI_PHY_833;
	volatile uint32_t DENALI_PHY_834;
	volatile uint32_t DENALI_PHY_835;
	volatile uint32_t DENALI_PHY_836;
	volatile uint32_t DENALI_PHY_837;
	volatile uint32_t DENALI_PHY_838;
	volatile uint32_t DENALI_PHY_839;
	volatile uint32_t DENALI_PHY_840;
	volatile uint32_t DENALI_PHY_841;
	volatile uint32_t DENALI_PHY_842;
	volatile uint32_t DENALI_PHY_843;
	volatile uint32_t DENALI_PHY_844;
	volatile uint32_t DENALI_PHY_845;
	volatile uint32_t DENALI_PHY_846;
	volatile uint32_t DENALI_PHY_847;
	volatile uint32_t DENALI_PHY_848;
	volatile uint32_t DENALI_PHY_849;
	volatile uint32_t DENALI_PHY_850;
	volatile uint32_t DENALI_PHY_851;
	volatile uint32_t DENALI_PHY_852;
	volatile uint32_t DENALI_PHY_853;
	volatile uint32_t DENALI_PHY_854;
	volatile uint32_t DENALI_PHY_855;
	volatile uint32_t DENALI_PHY_856;
	volatile uint32_t DENALI_PHY_857;
	volatile uint32_t DENALI_PHY_858;
	volatile uint32_t DENALI_PHY_859;
	volatile uint32_t DENALI_PHY_860;
	volatile uint32_t DENALI_PHY_861;
	volatile uint32_t DENALI_PHY_862;
	volatile uint32_t DENALI_PHY_863;
	volatile uint32_t DENALI_PHY_864;
	volatile uint32_t DENALI_PHY_865;
	volatile uint32_t DENALI_PHY_866;
	volatile uint32_t DENALI_PHY_867;
	volatile uint32_t DENALI_PHY_868;
	volatile uint32_t DENALI_PHY_869;
	volatile uint32_t DENALI_PHY_870;
	volatile uint32_t DENALI_PHY_871;
	volatile uint32_t DENALI_PHY_872;
	volatile uint32_t DENALI_PHY_873;
	volatile uint32_t DENALI_PHY_874;
	volatile uint32_t DENALI_PHY_875;
	volatile uint32_t DENALI_PHY_876;
	volatile uint32_t DENALI_PHY_877;
	volatile uint32_t DENALI_PHY_878;
	volatile uint32_t DENALI_PHY_879;
	volatile uint32_t DENALI_PHY_880;
	volatile uint32_t DENALI_PHY_881;
	volatile uint32_t DENALI_PHY_882;
	volatile uint32_t DENALI_PHY_883;
	volatile uint32_t DENALI_PHY_884;
	volatile uint32_t DENALI_PHY_885;
	volatile uint32_t DENALI_PHY_886;
	volatile uint32_t DENALI_PHY_887;
	volatile uint32_t DENALI_PHY_888;
	volatile uint32_t DENALI_PHY_889;
	volatile uint32_t DENALI_PHY_890;
	volatile uint32_t DENALI_PHY_891;
	volatile uint32_t DENALI_PHY_892;
	volatile uint32_t DENALI_PHY_893;
	volatile uint32_t DENALI_PHY_894;
	volatile uint32_t DENALI_PHY_895;
	volatile uint32_t DENALI_PHY_896;
	volatile uint32_t DENALI_PHY_897;
	volatile uint32_t DENALI_PHY_898;
	volatile uint32_t DENALI_PHY_899;
	volatile uint32_t DENALI_PHY_900;
	volatile uint32_t DENALI_PHY_901;
	volatile uint32_t DENALI_PHY_902;
	volatile uint32_t DENALI_PHY_903;
	volatile uint32_t DENALI_PHY_904;
	volatile uint32_t DENALI_PHY_905;
	volatile uint32_t DENALI_PHY_906;
	volatile uint32_t DENALI_PHY_907;
	volatile char pad__5[0x1D0U];
	volatile uint32_t DENALI_PHY_1024;
	volatile uint32_t DENALI_PHY_1025;
	volatile uint32_t DENALI_PHY_1026;
	volatile uint32_t DENALI_PHY_1027;
	volatile uint32_t DENALI_PHY_1028;
	volatile uint32_t DENALI_PHY_1029;
	volatile uint32_t DENALI_PHY_1030;
	volatile uint32_t DENALI_PHY_1031;
	volatile uint32_t DENALI_PHY_1032;
	volatile uint32_t DENALI_PHY_1033;
	volatile uint32_t DENALI_PHY_1034;
	volatile uint32_t DENALI_PHY_1035;
	volatile uint32_t DENALI_PHY_1036;
	volatile uint32_t DENALI_PHY_1037;
	volatile uint32_t DENALI_PHY_1038;
	volatile uint32_t DENALI_PHY_1039;
	volatile uint32_t DENALI_PHY_1040;
	volatile uint32_t DENALI_PHY_1041;
	volatile uint32_t DENALI_PHY_1042;
	volatile uint32_t DENALI_PHY_1043;
	volatile uint32_t DENALI_PHY_1044;
	volatile uint32_t DENALI_PHY_1045;
	volatile uint32_t DENALI_PHY_1046;
	volatile uint32_t DENALI_PHY_1047;
	volatile uint32_t DENALI_PHY_1048;
	volatile uint32_t DENALI_PHY_1049;
	volatile uint32_t DENALI_PHY_1050;
	volatile uint32_t DENALI_PHY_1051;
	volatile uint32_t DENALI_PHY_1052;
	volatile uint32_t DENALI_PHY_1053;
	volatile uint32_t DENALI_PHY_1054;
	volatile uint32_t DENALI_PHY_1055;
	volatile uint32_t DENALI_PHY_1056;
	volatile uint32_t DENALI_PHY_1057;
	volatile uint32_t DENALI_PHY_1058;
	volatile uint32_t DENALI_PHY_1059;
	volatile uint32_t DENALI_PHY_1060;
	volatile uint32_t DENALI_PHY_1061;
	volatile uint32_t DENALI_PHY_1062;
	volatile uint32_t DENALI_PHY_1063;
	volatile uint32_t DENALI_PHY_1064;
	volatile uint32_t DENALI_PHY_1065;
	volatile uint32_t DENALI_PHY_1066;
	volatile uint32_t DENALI_PHY_1067;
	volatile uint32_t DENALI_PHY_1068;
	volatile uint32_t DENALI_PHY_1069;
	volatile uint32_t DENALI_PHY_1070;
	volatile uint32_t DENALI_PHY_1071;
	volatile uint32_t DENALI_PHY_1072;
	volatile uint32_t DENALI_PHY_1073;
	volatile uint32_t DENALI_PHY_1074;
	volatile uint32_t DENALI_PHY_1075;
	volatile char pad__6[0x330U];
	volatile uint32_t DENALI_PHY_1280;
	volatile uint32_t DENALI_PHY_1281;
	volatile uint32_t DENALI_PHY_1282;
	volatile uint32_t DENALI_PHY_1283;
	volatile uint32_t DENALI_PHY_1284;
	volatile uint32_t DENALI_PHY_1285;
	volatile uint32_t DENALI_PHY_1286;
	volatile uint32_t DENALI_PHY_1287;
	volatile uint32_t DENALI_PHY_1288;
	volatile uint32_t DENALI_PHY_1289;
	volatile uint32_t DENALI_PHY_1290;
	volatile uint32_t DENALI_PHY_1291;
	volatile uint32_t DENALI_PHY_1292;
	volatile uint32_t DENALI_PHY_1293;
	volatile uint32_t DENALI_PHY_1294;
	volatile uint32_t DENALI_PHY_1295;
	volatile uint32_t DENALI_PHY_1296;
	volatile uint32_t DENALI_PHY_1297;
	volatile uint32_t DENALI_PHY_1298;
	volatile uint32_t DENALI_PHY_1299;
	volatile uint32_t DENALI_PHY_1300;
	volatile uint32_t DENALI_PHY_1301;
	volatile uint32_t DENALI_PHY_1302;
	volatile uint32_t DENALI_PHY_1303;
	volatile uint32_t DENALI_PHY_1304;
	volatile uint32_t DENALI_PHY_1305;
	volatile uint32_t DENALI_PHY_1306;
	volatile uint32_t DENALI_PHY_1307;
	volatile uint32_t DENALI_PHY_1308;
	volatile uint32_t DENALI_PHY_1309;
	volatile uint32_t DENALI_PHY_1310;
	volatile uint32_t DENALI_PHY_1311;
	volatile uint32_t DENALI_PHY_1312;
	volatile uint32_t DENALI_PHY_1313;
	volatile uint32_t DENALI_PHY_1314;
	volatile uint32_t DENALI_PHY_1315;
	volatile uint32_t DENALI_PHY_1316;
	volatile uint32_t DENALI_PHY_1317;
	volatile uint32_t DENALI_PHY_1318;
	volatile uint32_t DENALI_PHY_1319;
	volatile uint32_t DENALI_PHY_1320;
	volatile uint32_t DENALI_PHY_1321;
	volatile uint32_t DENALI_PHY_1322;
	volatile uint32_t DENALI_PHY_1323;
	volatile uint32_t DENALI_PHY_1324;
	volatile uint32_t DENALI_PHY_1325;
	volatile uint32_t DENALI_PHY_1326;
	volatile uint32_t DENALI_PHY_1327;
	volatile uint32_t DENALI_PHY_1328;
	volatile uint32_t DENALI_PHY_1329;
	volatile uint32_t DENALI_PHY_1330;
	volatile uint32_t DENALI_PHY_1331;
	volatile uint32_t DENALI_PHY_1332;
	volatile uint32_t DENALI_PHY_1333;
	volatile uint32_t DENALI_PHY_1334;
	volatile uint32_t DENALI_PHY_1335;
	volatile uint32_t DENALI_PHY_1336;
	volatile uint32_t DENALI_PHY_1337;
	volatile uint32_t DENALI_PHY_1338;
	volatile uint32_t DENALI_PHY_1339;
	volatile uint32_t DENALI_PHY_1340;
	volatile uint32_t DENALI_PHY_1341;
	volatile uint32_t DENALI_PHY_1342;
	volatile uint32_t DENALI_PHY_1343;
	volatile uint32_t DENALI_PHY_1344;
	volatile uint32_t DENALI_PHY_1345;
	volatile uint32_t DENALI_PHY_1346;
	volatile uint32_t DENALI_PHY_1347;
	volatile uint32_t DENALI_PHY_1348;
	volatile uint32_t DENALI_PHY_1349;
	volatile uint32_t DENALI_PHY_1350;
	volatile uint32_t DENALI_PHY_1351;
	volatile uint32_t DENALI_PHY_1352;
	volatile uint32_t DENALI_PHY_1353;
	volatile uint32_t DENALI_PHY_1354;
	volatile uint32_t DENALI_PHY_1355;
	volatile uint32_t DENALI_PHY_1356;
	volatile uint32_t DENALI_PHY_1357;
	volatile uint32_t DENALI_PHY_1358;
	volatile uint32_t DENALI_PHY_1359;
	volatile uint32_t DENALI_PHY_1360;
	volatile uint32_t DENALI_PHY_1361;
	volatile uint32_t DENALI_PHY_1362;
	volatile uint32_t DENALI_PHY_1363;
	volatile uint32_t DENALI_PHY_1364;
	volatile uint32_t DENALI_PHY_1365;
	volatile uint32_t DENALI_PHY_1366;
	volatile uint32_t DENALI_PHY_1367;
	volatile uint32_t DENALI_PHY_1368;
	volatile uint32_t DENALI_PHY_1369;
	volatile uint32_t DENALI_PHY_1370;
	volatile uint32_t DENALI_PHY_1371;
	volatile uint32_t DENALI_PHY_1372;
	volatile uint32_t DENALI_PHY_1373;
	volatile uint32_t DENALI_PHY_1374;
	volatile uint32_t DENALI_PHY_1375;
	volatile uint32_t DENALI_PHY_1376;
	volatile uint32_t DENALI_PHY_1377;
	volatile uint32_t DENALI_PHY_1378;
	volatile uint32_t DENALI_PHY_1379;
	volatile uint32_t DENALI_PHY_1380;
	volatile uint32_t DENALI_PHY_1381;
	volatile uint32_t DENALI_PHY_1382;
	volatile uint32_t DENALI_PHY_1383;
	volatile uint32_t DENALI_PHY_1384;
	volatile uint32_t DENALI_PHY_1385;
	volatile uint32_t DENALI_PHY_1386;
	volatile uint32_t DENALI_PHY_1387;
	volatile uint32_t DENALI_PHY_1388;
	volatile uint32_t DENALI_PHY_1389;
	volatile uint32_t DENALI_PHY_1390;
	volatile uint32_t DENALI_PHY_1391;
	volatile uint32_t DENALI_PHY_1392;
	volatile uint32_t DENALI_PHY_1393;
	volatile uint32_t DENALI_PHY_1394;
	volatile uint32_t DENALI_PHY_1395;
	volatile uint32_t DENALI_PHY_1396;
	volatile uint32_t DENALI_PHY_1397;
	volatile uint32_t DENALI_PHY_1398;
	volatile uint32_t DENALI_PHY_1399;
	volatile uint32_t DENALI_PHY_1400;
	volatile uint32_t DENALI_PHY_1401;
	volatile uint32_t DENALI_PHY_1402;
	volatile uint32_t DENALI_PHY_1403;
	volatile uint32_t DENALI_PHY_1404;
	volatile uint32_t DENALI_PHY_1405;
	volatile uint32_t DENALI_PHY_1406;
	volatile uint32_t DENALI_PHY_1407;
	volatile uint32_t DENALI_PHY_1408;
	volatile uint32_t DENALI_PHY_1409;
	volatile uint32_t DENALI_PHY_1410;
	volatile uint32_t DENALI_PHY_1411;
	volatile uint32_t DENALI_PHY_1412;
	volatile uint32_t DENALI_PHY_1413;
	volatile uint32_t DENALI_PHY_1414;
	volatile uint32_t DENALI_PHY_1415;
	volatile uint32_t DENALI_PHY_1416;
	volatile uint32_t DENALI_PHY_1417;
	volatile uint32_t DENALI_PHY_1418;
	volatile uint32_t DENALI_PHY_1419;
	volatile uint32_t DENALI_PHY_1420;
	volatile uint32_t DENALI_PHY_1421;
	volatile uint32_t DENALI_PHY_1422;
} lpddr4_ctlregs;

#endif /* REG_LPDDR4_CTL_REGS_H_ */
