/* SPDX-License-Identifier: GPL-2.0+ */
/* Copyright (c) 2019 Intel Corporation */

#ifndef _TANGIER_IOMAP_H
#define _TANGIER_IOMAP_H

#define MCFG_BASE_ADDRESS	0x3f500000
#define MCFG_BASE_SIZE		0x00100000

#endif	/* _TANGIER_IOMAP_H */
