# SPDX-License-Identifier:      GPL-2.0+

# $MIN_FILE is the name of the 20KB file in the file system image
MIN_FILE='testfile'

# $SMALL_FILE is the name of the 1MB file in the file system image
SMALL_FILE='1MB.file'

# $MEDIUM_FILE is the name of the 10MB file in the file system image
MEDIUM_FILE='10MB.file'

# $BIG_FILE is the name of the 2.5GB file in the file system image
BIG_FILE='2.5GB.file'

ADDR=0x01000008
LENGTH=0x00100000
