/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Álvaro Fernández Rojas <noltari@gmail.com>
 */

#ifndef __DT_BINDINGS_POWER_DOMAIN_BCM6328_H
#define __DT_BINDINGS_POWER_DOMAIN_BCM6328_H

#define BCM6328_PWR_ADSL2_MIPS	0
#define BCM6328_PWR_ADSL2_PHY	1
#define BCM6328_PWR_ADSL2_AFE	2
#define BCM6328_PWR_SAR		3
#define BCM6328_PWR_PCM		4
#define BCM6328_PWR_USBD	5
#define BCM6328_PWR_USBH	6
#define BCM6328_PWR_PCIE	7
#define BCM6328_PWR_ROBOSW	8
#define BCM6328_PWR_EPHY	9

#endif /* __DT_BINDINGS_POWER_DOMAIN_BCM6328_H */
