/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Renesas SuperH MMCIF driver.
 *
 * Copyright (C)  2012 Nobuhiro Iwamatsu <nobuhiro.iwamatsu.yj@renesas.com>
 * Copyright (C)  2012 Renesas Solutions Corp.
 *
 */
#ifndef _SH_MMC_H_
#define _SH_MMC_H_

int mmcif_mmc_init(void);

#endif /* _SH_MMC_H_ */
