/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * board.h
 *
 * Copyright (C) 2013 Lothar Felten <lothar.felten@gmail.com>
 */

#ifndef _BOARD_H_
#define _BOARD_H_

void enable_uart0_pin_mux(void);
void enable_board_pin_mux(void);

#endif
