/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (c) 2019 Microsemi Corporation
 */

#ifndef _LUTON_DATA_H_
#define _LUTON_DATA_H_

#define SERDES6G(x)     (x)
#define SERDES6G_MAX    SERDES6G(5)
#define SERDES_MAX      (SERDES6G_MAX)

/* similar with phy_interface_t */
#define PHY_MODE_SGMII  2
#define PHY_MODE_QSGMII 4

#endif
