/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * keystone: i2c driver definitions
 *
 * (C) Copyright 2014
 *     Texas Instruments Incorporated, <www.ti.com>
 */
#ifndef _I2C_DEFS_H_
#define _I2C_DEFS_H_

#define I2C0_BASE		0x02530000
#define I2C1_BASE		0x02530400
#define I2C2_BASE		0x02530800
#define I2C_BASE		I2C0_BASE

#endif
