/* SPDX-License-Identifier: GPL-2.0 */

#define trace_cdns3_prepare_trb(a, b)
#define trace_cdns3_doorbell_ep0(a, b)
#define trace_cdns3_ctrl_req(a)
#define trace_cdns3_complete_trb(a, b)
#define trace_cdns3_ep0_irq(a, b)
#define trace_cdns3_gadget_giveback(a)
#define trace_cdns3_free_aligned_request(a)
#define trace_cdns3_prepare_aligned_request(a)
#define trace_cdns3_ring(a)
#define trace_cdns3_doorbell_epx(a, b)
#define trace_cdns3_request_handled(a, b, c)
#define trace_cdns3_epx_irq(a, b)
#define trace_cdns3_usb_irq(a, b)
#define trace_cdns3_alloc_request(a)
#define trace_cdns3_free_request(a)
#define trace_cdns3_gadget_ep_enable(a)
#define trace_cdns3_gadget_ep_disable(a)
#define trace_cdns3_ep0_queue(a, b)
#define trace_cdns3_ep0_dequeue(a)
#define trace_cdns3_ep_queue(a)
#define trace_cdns3_ep_dequeue(a)
#define trace_cdns3_halt(a, b, c)
#define trace_cdns3_wa1(a, b)
#define trace_cdns3_wa2(a, b)
