/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2010, STMicroelectronics - All Rights Reserved
 * Author(s): Vipin Kumar, <vipin.kumar@st.com> for STMicroelectronics.
 */

static inline unsigned long get_macb_pclk_rate(unsigned int dev_id)
{
	return 83000000;
}
