/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2014 Freescale Semiconductor, Inc.
 */

#ifndef __ASM_ARCH_SPL_H__
#define __ASM_ARCH_SPL_H__

#define BOOT_DEVICE_NONE	0
#define BOOT_DEVICE_XIP		1
#define BOOT_DEVICE_XIPWAIT	2
#define BOOT_DEVICE_NAND	3
#define BOOT_DEVICE_ONENAND	4
#define BOOT_DEVICE_MMC1	5
#define BOOT_DEVICE_MMC2	6
#define BOOT_DEVICE_MMC2_2	7
#define BOOT_DEVICE_SPI		10

#endif	/* __ASM_ARCH_SPL_H__ */
