/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2017 Rockchip Electronics Co., Ltd
 */

#ifndef __EVB_RK3128_H
#define __EVB_RK3128_H

#include <configs/rk3128_common.h>

#define CONFIG_SYS_MMC_ENV_DEV 0

#endif
