// SPDX-License-Identifier: GPL-2.0
/*
 * USBSS device controller driver Trace Support
 *
 * Copyright (C) 2018-2019 Cadence.
 *
 * Author: Pawel Laszczak <pawell@cadence.com>
 */

#define CREATE_TRACE_POINTS
#include "trace.h"
