/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2014 Freescale Semiconductor, Inc.
 */

#ifndef __FSL_DDRC_VER_H
#define __FSL_DDRC_VER_H

/*
 * Only the versions with distinct features or registers are listed here.
 */
#define FSL_DDR_VER_4_4 44
#define FSL_DDR_VER_4_6 46
#define FSL_DDR_VER_4_7	47
#define FSL_DDR_VER_5_0	50

#endif /* __FSL_DDRC_VER_H */
