/* SPDX-License-Identifier: <SPDX License Expression> */
/*
 * Copyright 2019 Broadcom
 *
 */

#ifndef __IPROC_SDHCI_H
#define __IPROC_SDHCI_H

int iproc_sdhci_init(int dev_index, u32 quirks);

#endif
