/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2011
 * Yuri Tikhonov, Emcraft Systems, yur@emcraft.com
 *
 * (C) Copyright 2015
 * Kamil Lulko, <kamil.lulko@gmail.com>
 */

#ifndef _STM32_GPIO_H_
#define _STM32_GPIO_H_

#include <asm/arch-stm32/gpio.h>

#endif /* _STM32_GPIO_H_ */
