/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2019 Vasily Khoruzhick <anarsoul@gmail.com>
 */

#ifndef __ROCKPRO64_RK3399_H
#define __ROCKPRO64_RK3399_H

#include <configs/rk3399_common.h>

#define CONFIG_SYS_MMC_ENV_DEV 0

#define SDRAM_BANK_SIZE			(2UL << 30)

#endif
